/*-
 * %%----------------------------------------------------------------------------------------------
 * Archive ouverte UNIGE - AoU Model - BibTexEscaper.java
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2022 - 2023 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

package ch.unige.aou.model.access;

import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class BibTexEscaper {

  private static final Map<Character, String> ESCAPE_MAP = new HashMap<>();

  static {
    ESCAPE_MAP.put('*', "\\*");
    ESCAPE_MAP.put('#', "\\#");
    ESCAPE_MAP.put('@', "\\@");
    ESCAPE_MAP.put('!', "\\!");
    ESCAPE_MAP.put('&', "\\&"); // Escaping the & character
  }

  /**
   * Needed to escape certains characters and uppercase words only for title property
   * @param input
   * @param isTitleProperty
   * @return
   */
  public static String escapeForBibTeX(String input, Boolean isTitleProperty) {
    StringBuilder strBuilder = new StringBuilder();
    if (isTitleProperty) {
      if (containsUpperCaseCharacter(input)) {
        strBuilder.append(escapeUppercaseWord(input));
      } else {
        strBuilder.append(escapeSpecialCharacters(input)).append(" ");
      }
    } else {
      String[] words = input.split("\\s+");
      for (String word : words) {
        strBuilder.append(escapeSpecialCharacters(word)).append(" ");
      }
    }
    return strBuilder.toString().trim();
  }

  private static boolean containsUpperCaseCharacter(String word) {
    // Check if the word is all uppercase and has more than one character
    Pattern pattern = Pattern.compile("(?=.*[A-Z].*)^.{1,}[^A-Z]*[A-Z].*");

    Matcher matcher = pattern.matcher(word);
    return word.length() > 1 && matcher.matches();
  }

  private static boolean hasAnyUpperCaseCharacter(String word) {
    return word.chars().anyMatch(Character::isUpperCase);
  }

  private static String escapeUppercaseWord(String word) {
    // Method to escape between {} any word containing an uppercase character that is not the start of the title.
    StringBuilder stringBuilder = new StringBuilder();
    //Separate each word
    String[] parts = word.split("[\\s,;.:?!\"/']+");
    int wordIndex = 0;
    for(String part: parts) {
      if (parts[0].equals(part) || !hasAnyUpperCaseCharacter(part)) {
        stringBuilder.append(escapeSpecialCharacters(part));
      } else {
        // contains at least one uppercase and it's not the first word, need to check each of the words separated by "-" which are escaped individually in case
        // of uppercase characters. Ex, anti-SARS-COV-2 gives anti-{SARS}-{COV}-2
        String[] strHyphenSplitted = part.split("-");
        int counterHyphens = 0;
        for (String strHyphenSplit:strHyphenSplitted) {
          if (hasAnyUpperCaseCharacter(strHyphenSplit)) {
            stringBuilder.append("{" + strHyphenSplit + "}");
          } else {
            stringBuilder.append(escapeSpecialCharacters(strHyphenSplit));
          }
          //append "-" separator between each of the parts until reaching the end
          if (counterHyphens != (strHyphenSplitted.length -1) ) {
            stringBuilder.append("-");
          }
          counterHyphens++;
        }
      }
      //add separator character if not reached the end
      if (stringBuilder.length() < word.length()) {
        char c = word.charAt(wordIndex + part.length());
        stringBuilder.append(c);
        wordIndex = wordIndex + part.length() + 1;
        // Check if there is any more separator used to split each word and add all of them.
        // Example: C., & Marcel should be {C}., \& {Marcel}
        boolean addMoreSeparatorCharacters = false;
        do {
          if (wordIndex < word.length()) {
            char charAfter = word.charAt(wordIndex);
            if (charAfter == '.' || charAfter == ',' || charAfter == ';' || charAfter == ':'
                    || charAfter == '/' || charAfter == '?' || charAfter == '!' || charAfter == '"' || charAfter == ' ') {
              stringBuilder.append(charAfter);
              wordIndex++;
              addMoreSeparatorCharacters = true;
            } else {
              addMoreSeparatorCharacters = false;
            }
          }
        } while (addMoreSeparatorCharacters);
      }
    }
    return stringBuilder.toString();
  }


  private static String escapeSpecialCharacters(String input) {
    StringBuilder escapedText = new StringBuilder();

    for (char c : input.toCharArray()) {
      String escapedChar = ESCAPE_MAP.get(c);
      if (escapedChar != null) {
        escapedText.append(escapedChar);
      } else {
        escapedText.append(c);
      }
    }
    return escapedText.toString();
  }

}

